@@ -12,7 +12,6 @@ import (
 	"github.com/mickael-menu/zk/internal/util"
 	"github.com/mickael-menu/zk/internal/util/errors"
 	"github.com/mickael-menu/zk/internal/util/fts5"
-	"github.com/mickael-menu/zk/internal/util/opt"
 	"github.com/mickael-menu/zk/internal/util/paths"
 	strutil "github.com/mickael-menu/zk/internal/util/strings"
 )
@@ -421,9 +420,7 @@ func (d *NoteDAO) expandMentionsIntoMatch(opts core.NoteFindOpts) (core.NoteFind
 	}
 
 	// Expand the mention queries in the match predicate.
-	match := opts.Match.String()
-	match += " " + strings.Join(mentionQueries, " OR ")
-	opts.Match = opt.NewString(match)
+	opts.Match = append(opts.Match, " ("+strings.Join(mentionQueries, " OR ")+") ")
 
 	return opts, nil
 }
@@ -448,7 +445,7 @@ func (d *NoteDAO) findRows(opts core.NoteFindOpts, selection noteSelection) (*sq
 	transitiveClosure := false
 	maxDistance := 0
 
-	setupLinkFilter := func(hrefs []string, direction int, negate, recursive bool) error {
+	setupLinkFilter := func(tableAlias string, hrefs []string, direction int, negate, recursive bool) error {
 		ids, err := d.findIdsByHrefs(hrefs, true /* allowPartialHrefs */)
 		if err != nil {
 			return err
@@ -463,27 +460,29 @@ func (d *NoteDAO) findRows(opts core.NoteFindOpts, selection noteSelection) (*sq
 		if recursive {
 			transitiveClosure = true
 			linksSrc = "transitive_closure"
+			additionalOrderTerms = append(additionalOrderTerms, tableAlias+".distance")
 		}
 
 		if !negate {
 			if direction != 0 {
-				snippetCol = "GROUP_CONCAT(REPLACE(l.snippet, l.title, '<zk:match>' || l.title || '</zk:match>'), '\x01')"
+				snippetCol = fmt.Sprintf("GROUP_CONCAT(REPLACE(%s.snippet, %[1]s.title, '<zk:match>' || %[1]s.title || '</zk:match>'), '\x01')", tableAlias)
 			}
 
 			joinOns := make([]string, 0)
 			if direction <= 0 {
 				joinOns = append(joinOns, fmt.Sprintf(
-					"(n.id = l.target_id AND l.source_id IN %s)", idsList,
+					"(n.id = %[1]s.target_id AND %[1]s.source_id IN %[2]s)", tableAlias, idsList,
 				))
 			}
 			if direction >= 0 {
 				joinOns = append(joinOns, fmt.Sprintf(
-					"(n.id = l.source_id AND l.target_id IN %s)", idsList,
+					"(n.id = %[1]s.source_id AND %[1]s.target_id IN %[2]s)", tableAlias, idsList,
 				))
 			}
 
 			joinClauses = append(joinClauses, fmt.Sprintf(
-				"LEFT JOIN %s l ON %s",
+				"LEFT JOIN %[2]s %[1]s ON %[3]s",
+				tableAlias,
 				linksSrc,
 				strings.Join(joinOns, " OR "),
 			))
@@ -517,20 +516,26 @@ func (d *NoteDAO) findRows(opts core.NoteFindOpts, selection noteSelection) (*sq
 		return nil
 	}
 
-	if !opts.Match.IsNull() {
+	if 0 < len(opts.Match) {
 		switch opts.MatchStrategy {
 		case core.MatchStrategyExact:
-			whereExprs = append(whereExprs, `n.raw_content LIKE '%' || ? || '%' ESCAPE '\'`)
-			args = append(args, escapeLikeTerm(opts.Match.String(), '\\'))
+			for _, match := range opts.Match {
+				whereExprs = append(whereExprs, `n.raw_content LIKE '%' || ? || '%' ESCAPE '\'`)
+				args = append(args, escapeLikeTerm(match, '\\'))
+			}
 		case core.MatchStrategyFts:
 			snippetCol = `snippet(fts_match.notes_fts, 2, '<zk:match>', '</zk:match>', '…', 20)`
 			joinClauses = append(joinClauses, "JOIN notes_fts fts_match ON n.id = fts_match.rowid")
 			additionalOrderTerms = append(additionalOrderTerms, `bm25(fts_match.notes_fts, 1000.0, 500.0, 1.0)`)
-			whereExprs = append(whereExprs, "fts_match.notes_fts MATCH ?")
-			args = append(args, fts5.ConvertQuery(opts.Match.String()))
+			for _, match := range opts.Match {
+				whereExprs = append(whereExprs, "fts_match.notes_fts MATCH ?")
+				args = append(args, fts5.ConvertQuery(match))
+			}
 		case core.MatchStrategyRe:
-			whereExprs = append(whereExprs, "n.raw_content REGEXP ?")
-			args = append(args, opts.Match.String())
+			for _, match := range opts.Match {
+				whereExprs = append(whereExprs, "n.raw_content REGEXP ?")
+				args = append(args, match)
+			}
 			break
 		}
 	}
@@ -618,7 +623,7 @@ WHERE collection_id IN (SELECT id FROM collections t WHERE kind = '%s' AND (%s))
 	if opts.LinkedBy != nil {
 		filter := opts.LinkedBy
 		maxDistance = filter.MaxDistance
-		err := setupLinkFilter(filter.Hrefs, -1, filter.Negate, filter.Recursive)
+		err := setupLinkFilter("l_by", filter.Hrefs, -1, filter.Negate, filter.Recursive)
 		if err != nil {
 			return nil, err
 		}
@@ -627,7 +632,7 @@ WHERE collection_id IN (SELECT id FROM collections t WHERE kind = '%s' AND (%s))
 	if opts.LinkTo != nil {
 		filter := opts.LinkTo
 		maxDistance = filter.MaxDistance
-		err := setupLinkFilter(filter.Hrefs, 1, filter.Negate, filter.Recursive)
+		err := setupLinkFilter("l_to", filter.Hrefs, 1, filter.Negate, filter.Recursive)
 		if err != nil {
 			return nil, err
 		}
@@ -635,11 +640,11 @@ WHERE collection_id IN (SELECT id FROM collections t WHERE kind = '%s' AND (%s))
 
 	if opts.Related != nil {
 		maxDistance = 2
-		err := setupLinkFilter(opts.Related, 0, false, true)
+		err := setupLinkFilter("l_rel", opts.Related, 0, false, true)
 		if err != nil {
 			return nil, err
 		}
-		groupBy += " HAVING MIN(l.distance) = 2"
+		groupBy += " HAVING MIN(l_rel.distance) = 2"
 	}
 
 	if opts.Orphan {
@@ -687,8 +692,6 @@ WHERE collection_id IN (SELECT id FROM collections t WHERE kind = '%s' AND (%s))
 
 	// Credit to https://inviqa.com/blog/storing-graphs-database-sql-meets-social-network
 	if transitiveClosure {
-		orderTerms = append([]string{"l.distance"}, orderTerms...)
-
 		query += `WITH RECURSIVE transitive_closure(source_id, target_id, title, snippet, distance, path) AS (
     SELECT source_id, target_id, title, snippet,
            1 AS distance,
